/***************************************************************************
                          CdboAction.h  -  description
                             -------------------
	begin                : Wed Aug 14 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBOACTION_H
#define CDBOACTION_H

#include <qdict.h>
#include <qptrqueue.h>

#include <kio/job.h>


class KProcess;
class KConfig;
class KLibFactory;

class CdboActionHandler;


typedef QDict<QString> CdboParamsDict;

class CdboAction : public QObject{
friend class CdboActionHandler;
	Q_OBJECT
public:
	CdboAction(QObject *user, const char *name = 0);
	~CdboAction();
	
	virtual void performTask(CdboParamsDict* params);

	virtual bool tryToCancel();
	virtual void killCurrentTask();
	void setAutoDelete(bool doit) { autoDelete = doit; }
	bool isAutoDelete() { return autoDelete; }
	
	void removeTmpImage(QString image, QString dirName);
	void removeOneFile(QString fileName, bool showError = true);
	void removeOneDir(QString dirName, bool showError = true);
	
	bool initProcessForPiping(KProcess *proc, CdboParamsDict* params, CdboAction *owner);
	virtual bool filterOutput(int &, QString &) { return true; }
	
	static QString timeStamp();
	static QString timeString(long posSecs);

signals:
	void processDone(bool success);
	void failedToLaunchProcess();
	void processCancelled();

	void blockBtns(bool doIt);
	void status(QString msg);
	void output(int type, QString message);
	void percent(QString procId, int percentVal, QString optMess = QString::null);
	void percentInfo(QString procId, QString message);

	void actionDone(CdboAction*);
	
protected slots:
	virtual void receivedMessageSlot(KProcess *proc, char *buffer, int buflen);
	virtual void receivedErrMessageSlot(KProcess *proc, char *buffer, int buflen);
	void processDoneSlot(KProcess *);
	virtual void timerEvent(QTimerEvent *event);

	void cancelCurrentTask();
	virtual void processCancelledByUser();
	void emitBlockBtns(bool doIt) { emit blockBtns(doIt); }
	void emitStatus(QString msg) { emit status(msg); }
	void emitOutput(int type, QString message) { emit output(type, message); }
	void emitPercent(QString procId, int percentVal, QString optMess = QString::null){
		emit percent(procId, percentVal, optMess);
	}
	void emitPercentInfo(QString procId, QString message) { emit percentInfo(procId, message); }

	void emitActionDone();
	void ejectProcComplited(KProcess *);
	
protected:
	virtual void prepareForProcess(CdboParamsDict* params);
	virtual void initProcess(KProcess *proc, CdboParamsDict* params) = 0;
	void launchProcess(KProcess *proc);
	void debugProcess(KProcess *proc);
	void processOutput(int type, char *buffer, int buflen);
	
	virtual void processLaunched();
	virtual void processFailedToLaunch();
	virtual void processSucceded();
	virtual void processFailed();
	
	void internalError(QString errorText);
	
	virtual void resetSettings();
	virtual void resetConfig();
	virtual QString cancelMessage();

	void enableTimeout();
	void resetTimeout();
	
	bool sendSignalToProcess();
	bool requestReload(bool signalToProcess = true);
	bool requestCDR(bool signalToProcess = true, const QString &driveName = QString::null);
	void eject(const QString &driveName);
	void closeTray(const QString &driveName);
	
	QString paramVal(QString paramName, bool showError = true);
	bool boolParamVal(QString paramName, bool &value, bool showError = true);
	bool intParamVal(QString paramName, int &num, bool showError = true);
	bool listParamVal(QString paramName, QStringList &list, QString breakFlag = "\n", bool showError = true);
	void setParamVal(QString paramName, QString *paramValue);
	
	//members
	bool DEBUG_COMMANDS;
	bool DEBUG_OUTPUT;
	KProcess *currentProc;
	KConfig *config;
	CdboParamsDict *params;
	
	bool doingSomething;
	bool readyToLaunch;
	bool unlockUser;
	bool canceled;
	bool showCanceledMess;
	bool autoDelete;
	
	bool watchTime;
	int timeOutId;
	bool stillEjecting;

};

///////////////////////////
class CdboJobBasedAction : public CdboAction{
	Q_OBJECT

public:
	CdboJobBasedAction(QObject *user, const char *name);
	~CdboJobBasedAction();

	virtual void performTask(CdboParamsDict* params);
	virtual void killCurrentTask();

protected:
	virtual void launchJob(CdboParamsDict* params) = 0;
	virtual void initProcess(KProcess *, CdboParamsDict*){;}
	virtual void resetSettings();
	KIO::Job *curJob;

};


typedef QPtrQueue<CdboAction> CdboActionsQueue;


//////////////////////
class CdboActionHandler : public CdboAction{
	Q_OBJECT

public:
	CdboActionHandler(KLibFactory* factory, QObject *user, const char *name);
	~CdboActionHandler();

	virtual void killCurrentTask();

protected slots:
	virtual void slaveActionFailedToLaunch();
	virtual void slaveActionCancelled();
	virtual void slaveActionDone(bool success);

	void lunchSlaveAction();
	void begin();
	
protected:
	virtual void prepareForProcess(CdboParamsDict* params);
	bool createAction(QString type, KLibFactory* factory = 0);
	virtual void resetSettings();
	void removeAllSlaves();

	KLibFactory* parentFactory;
	CdboActionsQueue slaveQueue;
	CdboAction* slaveAction;
	
	int copiesWanted;
	int copiesDone;

};
#endif
