/***************************************************************************
                          CdboViewPart.h  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBOVIEWPART_H
#define CDBOVIEWPART_H

#include <kparts/part.h>
#include <kparts/browserextension.h>
#include <kparts/genericfactory.h>
#include <kio/job.h>

class KProcess;

class CdboViewWidgetBase;

class CdboViewPart : public KParts::ReadWritePart{
Q_OBJECT
public:
	CdboViewPart( QWidget *parentWidget, const char *widgetName,
                  QObject *parent, const char *name=0, const QStringList& args = QStringList());
	virtual ~CdboViewPart();
	
	virtual bool closeURL();
	virtual void setModified ( bool modified );
	QString type(){ return m_type; }
	void setType(const QString& type){ m_type = type; }
	QString icon(){ return m_icon; }
	void setIcon(const QString& icon){ m_icon = icon; }
	QString lastStatus(){ return m_lastStatus; }
	virtual QString shortCaption(){ return m_shortCaption; }
	virtual void setShortCaption(const QString & newCaption){ m_shortCaption = newCaption; }
	virtual QString caption(){ return m_caption; }
	virtual void setCaption(const QString & newCaption){ m_caption = newCaption; }
	
public slots:
	virtual bool openURL(const KURL&);
	virtual bool saveFile();
	bool saveAs();
	virtual void addItems(KURL::List &){}
	virtual void readOptions();

signals:
	void processStarted();
	void processDone();
	void haveStatus(const QString &);
	void shortCaptionChanged(KParts::Part*);
	void captionChanged(KParts::Part*);
	
protected slots:
	void createCDSlot();
	void settingsSlot();
	void kcmshellDone(KProcess *proc);
	
	void dummySlot(){}
	void jobDone(KIO::Job *);
	void cannotFitError(const QString &fname);
	void emitLimitReached();
	void limitReachedError();
	void showCannotFitError();
	void emitCaptionChanged(){ emit captionChanged(this); }
	void emitShortCaptionChanged(){ emit shortCaptionChanged(this); }
	void emitStatus(const QString &status){ m_lastStatus = status; emit haveStatus(m_lastStatus);  }
	void resetStatus(){ m_lastStatus = ""; emit haveStatus(m_lastStatus);  }

protected:
	virtual bool isEmpty(){ return true; }
	virtual int getSize() {return 0; }
	virtual void createCD() {}
	virtual void saveToFile(KConfig *) {}
	
	void setupActions();
	CdboViewWidgetBase* viewBase(){ return m_viewBase; }
	
	bool mountDevice(QString &drive, bool &needToUmount);
	void umountDevice(QString &drive);
	
	void popupSettings(QString type);
	
	bool checkDirSize();
	void showEmptyError();
	void proposeDeviceDetection();
	bool checkOverwrite(const QString &path);
	void invalidPathError(const QString &path);
	void localUrlsOnlyError(const QString &path);
	int shouldOverwrite(QString name);
	
	bool getNewFileName();
	
	QString settingTypes;

private:
	CdboViewWidgetBase *m_viewBase;
	QString m_type;
	QString m_icon;
	QString m_lastStatus;
	QString m_shortCaption;
	QString lastFailedName;
	QString m_caption;
	
	bool doingSomething;
};


#endif

